#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "eit.h"

static void timecmp(int *,int *,int *,
					int, int, int);

int parseEIThead(unsigned char *data, EIThead *h) {
	int boff = 0;

	memset(h, 0, sizeof(EIThead));

	h->table_id = getBit(data, &boff, 8);
	h->section_syntax_indicator = getBit(data, &boff, 1);
	h->reserved_future_use = getBit(data, &boff, 1);
	h->reserved1 = getBit(data, &boff, 2);
	h->section_length =getBit(data, &boff,12);
	h->service_id = getBit(data, &boff, 16);
	h->reserved2 = getBit(data, &boff, 2);
	h->version_number = getBit(data, &boff, 5);
	h->current_next_indicator = getBit(data, &boff, 1);
	h->section_number = getBit(data, &boff, 8);
	h->last_section_number = getBit(data, &boff, 8);
	h->transport_stream_id = getBit(data, &boff, 16);
	h->original_network_id = getBit(data, &boff, 16);
	h->segment_last_section_number = getBit(data, &boff, 8);
	h->last_table_id = getBit(data, &boff, 8);
  
	return 14;
}

int parseEITbody(unsigned char *data, EITbody *b) {
	int boff = 0;
	int tnum;
	char buf[4];

	memset(b, 0, sizeof(EITbody));

	b->event_id = getBit(data, &boff, 16);

	memcpy(b->start_time, data + boff / 8, 5);
	/* b->start_time = getBit(data, &boff, 40); */
	boff += 40;
	memcpy(b->duration, data + boff / 8, 3);
	/* b->duration = getBit(data, &boff, 24); */
	boff += 24;
	b->running_status = getBit(data, &boff, 3);
	b->free_CA_mode = getBit(data, &boff, 1);
	b->descriptors_loop_length = getBit(data, &boff, 12);

	/* Ѵ */
	tnum = (b->start_time[0] & 0xFF) << 8 | (b->start_time[1] & 0xFF);
  
	b->yy = (tnum - 15078.2) / 365.25;
	b->mm = ((tnum - 14956.1) - (int)(b->yy * 365.25)) / 30.6001;
	b->dd = (tnum - 14956) - (int)(b->yy * 365.25) - (int)(b->mm * 30.6001);

	if(b->dd == 0) {
		printf("aa");
	}

	if(b->mm == 14 || b->mm == 15) {
		b->yy += 1;
		b->mm = b->mm - 1 - (1 * 12);
	} else {
		b->mm = b->mm - 1;
	}

	b->yy += 1900;
  
	sprintf(buf, "%x", b->start_time[2]);
	b->hh = atoi(buf);
	sprintf(buf, "%x", b->start_time[3]);
	b->hm = atoi(buf);
	sprintf(buf, "%x", b->start_time[4]);
	b->ss = atoi(buf);

	sprintf(buf, "%x", b->duration[0]);
	b->dhh = atoi(buf);
	sprintf(buf, "%x", b->duration[1]);
	b->dhm = atoi(buf);
	sprintf(buf, "%x", b->duration[2]);
	b->dss = atoi(buf);
  
	return 12;
}

int parseSEVTdesc(unsigned char *data, SEVTdesc *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(SEVTdesc));

	desc->descriptor_tag = getBit(data, &boff, 8);
	if((desc->descriptor_tag & 0xFF) != 0x4D) {
		return -1;
	}
	desc->descriptor_length = getBit(data, &boff, 8);
	memcpy(desc->ISO_639_language_code, data + boff / 8, 3);
	/* desc->ISO_639_language_code = getBit(data, &boff, 24); */
	boff += 24;
	desc->event_name_length = getBit(data, &boff, 8);
	getStr(desc->event_name, data, &boff, desc->event_name_length);
	desc->text_length = getBit(data, &boff, 8);
	getStr(desc->text, data, &boff, desc->text_length);

	return desc->descriptor_length + 2;
}

int parseEEVTDhead(unsigned char *data, EEVTDhead *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(EEVTDhead));

	desc->descriptor_tag = getBit(data, &boff, 8);
	if((desc->descriptor_tag & 0xFF) != 0x4E) {
		return -1;
	}
	desc->descriptor_length = getBit(data, &boff, 8);
	desc->descriptor_number = getBit(data, &boff, 4);
	desc->last_descriptor_number = getBit(data, &boff, 4);
	memcpy(desc->ISO_639_language_code, data + boff / 8, 3);
	/* desc->ISO_639_language_code = getBit(data, &boff, 24); */
	boff += 24;

	desc->length_of_items = getBit(data, &boff, 8);

	return 7;
}

int parseEEVTDitem(unsigned char *data, EEVTDitem *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(EEVTDitem));

	desc->item_description_length = getBit(data, &boff, 8);
	getStr(desc->item_description, data, &boff, desc->item_description_length);

	desc->item_length = getBit(data, &boff, 8);
	memcpy(desc->item, data + (boff / 8), desc->item_length);
	/* getStr(desc->item, data, &boff, desc->item_length); */

	return desc->item_description_length + desc->item_length + 2;
}

int parseEEVTDtail(unsigned char *data, EEVTDtail *desc) {
	int boff = 0;
  
	memset(desc, 0, sizeof(EEVTDtail));

	desc->text_length = getBit(data, &boff, 8);
	getStr(desc->text, data, &boff, desc->text_length);

	return desc->text_length + 1;
}

int checkEEVTDitem(EEVTDitem *save, EEVTDitem *new, int descriptor_number) {

	EEVTDitem swap;
	int boff = 0;

	if(new == NULL) {
		if(save->item_length != 0) {
			swap = *save;
			getStr(save->item, (unsigned char*)swap.item, &boff, swap.item_length);
			return 1;
		} else {
			return 0;
		}
	}

	if(new->item_description_length == 0) {
		/* ³ ¸ */
		memcpy(save->item + save->item_length, new->item, new->item_length);
		save->item_length += new->item_length;
		return 0;
	} else {
		/* ֥졼saveоݤˤ롣save򥯥ꥢ? */
		if(save->item_length != 0) {
			/* Ѥߤ */
			swap = *save;
			getStr(save->item, (unsigned char*)swap.item, &boff, swap.item_length);
			swap = *new;
			*new = *save;
			*save = swap;
			save->descriptor_number = descriptor_number;
		} else {
			*save = *new;
			save->descriptor_number = descriptor_number;
			return 0;
		}
	}

	return 1;
}


void dumpEIT(FILE *out, unsigned char *ptr, int mode) {

	EIThead  eith;
	EITbody  eitb;
	SEVTdesc sevtd;

	EEVTDhead eevthead;
	EEVTDitem eevtitem;
	EEVTDtail eevttail;

	EEVTDitem save_eevtitem;


	int len = 0;
	int loop_len = 0;
	int loop_blen = 0;
	int loop_elen = 0;

	int ehh, emm, ess;

	/* EIT */
	len = parseEIThead(ptr, &eith); 

	/*
	  if(eith.service_id == 32206) {
	  printf("0x%x\n", eith.table_id);
	  return;
	  }
	*/

	ptr += len;
	loop_len = eith.section_length - (len - 3 + 4); // 3϶̥إåĹ 4CRC
	while(loop_len > 0) {
		/* Ϣ³ĥ٥Ȥϡɤ㤭̤줷
		   ʬ䤵褦Ϣ³ɤϡitem_description_length
		   ꤵƤ뤫ɤȽǤǤ褦 */
		memset(&save_eevtitem, 0, sizeof(EEVTDitem));

		len = parseEITbody(ptr, &eitb);
		ptr += len;
		loop_len -= len;
    
		/* printf("evtid:%d\n", eitb.event_id); */
    
		loop_blen = eitb.descriptors_loop_length;
		loop_len -= loop_blen;
		while(loop_blen > 0) {

			len = parseSEVTdesc(ptr, &sevtd);
			if(len > 0) {

				/*
				  if(eith.service_id == 19304 && 
				  eitb.event_id == 46564) {
				  printf("aa");
				  }
				*/

				ehh = eitb.hh;
				emm = eitb.hm;
				ess = eitb.ss;

				timecmp(&ehh, &emm, &ess,
						eitb.dhh, eitb.dhm, eitb.dss);

				if(mode == 0) { /* sort format */
					fprintf(out, "SEVT,%d,%04d/%02d/%02d,%02d:%02d:%02d,%02d:%02d:%02d,%d,%s\n", 
							eith.service_id,
							eitb.yy, eitb.mm, eitb.dd,
							eitb.hh, eitb.hm, eitb.ss,
							ehh,emm,ess,
							/* eitb.dhh, eitb.dhm, eitb.dss, */
							eitb.dhh * 3600 + eitb.dhm * 60 + eitb.dss,
							sevtd.event_name);
				}

				if(mode == 1) { /* long format */
					fprintf(out, "SEVT,%d,%d,-1,%04d/%02d/%02d,%02d:%02d:%02d,%02d:%02d:%02d,%02d:%02d:%02d,%d,%s,%s\n", 
							eith.service_id,
							eitb.event_id,
							eitb.yy, eitb.mm, eitb.dd,
							eitb.hh, eitb.hm, eitb.ss,
							ehh,emm,ess,
							eitb.dhh, eitb.dhm, eitb.dss,
							eitb.dhh * 3600 + eitb.dhm * 60 + eitb.dss,
							sevtd.event_name,
							sevtd.text);
				}

			} else {
				len = parseEEVTDhead(ptr, &eevthead);

				/*
				  if(eith.service_id == 19304 && 
				  eitb.event_id == 46564) {
				  printf("aa");
				  }
				*/


				if(len > 0) {
					ptr += len;
					loop_blen -= len;

					loop_elen = eevthead.length_of_items;
					loop_len -= loop_elen;
					while(loop_elen > 0) {
						len = parseEEVTDitem(ptr, &eevtitem);

						ptr += len;
						loop_elen -= len;
						loop_blen -= len;

						if(checkEEVTDitem(&save_eevtitem, &eevtitem, 
										  eevthead.descriptor_number)) {
							if(mode == 1) { /* long format */
								fprintf(out, "EEVT,%d,%d,%d,%s,%s\n", 
										eith.service_id,
										eitb.event_id,
										eevtitem.descriptor_number, /*  */
										eevtitem.item_description,
										eevtitem.item);
							}
						}
					}

					len = parseEEVTDtail(ptr, &eevttail);

					if(mode == 1) { /* long format */
						fprintf(out, "EEVTt,%d,%d,%d,%s\n", 
								eith.service_id,
								eitb.event_id,
								eevthead.descriptor_number,
								eevttail.text);
					}

				} else {
					len = parseOTHERdesc(ptr);
				}
			}
			ptr += len;
			loop_blen -= len;
		}
		/* ǸΥ֥졼å */
    
		if(checkEEVTDitem(&save_eevtitem, NULL, 0)) {
			if(mode == 1) { /* long format */
				fprintf(out, "EEVT,%d,%d,%d,%s,%s\n", 
						eith.service_id,
						eitb.event_id,
						save_eevtitem.descriptor_number,
						save_eevtitem.item_description,
						save_eevtitem.item);
			}
		}
	}

	return;
}

void timecmp(int *thh, int *tmm, int *tss,
			 int dhh, int dmm, int dss) {

	int ama;

	*tss += dss;
	ama = *tss % 60;
	*tmm += (*tss / 60);
	*tss = ama;

	*tmm += dmm;
	ama   = *tmm % 60;
	*thh += (*tmm / 60);
	*tmm  = ama;

	*thh += dhh;

}
