#include <stdio.h>
#include <stdlib.h>
#include <pthread.h>

#include <string.h>
#include <unistd.h>
#include <math.h>

#include "friio.h"

static void UsbSendVendorRequest(uint16_t wValue, uint16_t wIndex);
static int UsbInitalize(const arginfo *ainfo);
static int UsbProcFixInitA();
static int UsbProcFixInitB();
static int UsbProcStreamInit();
static int UsbProcLED(const uint8_t bType);
static int UsbTerminate();
static void SetChannel(const uint8_t bCh);
static float GetSignalLevel();

static int UsbProcFixInitA_Black();
static int UsbProcFixInitB_Black();
static void SetChannel_Black(uint32_t dwCh);
static float GetSignalLevel_Black();

static void readts(const char *fname, int recsec, int packetcount);
typedef struct _th_arg {
	int  *endflag;
	int   recsec;
} th_arg;
static void *endcheck(void *arg);

static void arginit(int arc, char *argv[], arginfo *ainfo);
#define PRINTUSAGE fprintf(stderr, "usage: %s [-b] [-r rtpri] [-p packetcount] [-v venderid] [-i id] [-n num] [-w wait] channel recsec destfile\n", argv[0]);


/*
  %s channel recsec destfile\n

  -p packetcount
  -v venderid
  -i id
  -n num
  -w wait
  -b bs/cs⡼
  -r rtpri RealTime饹ץ饤ƥ

*/


int 
main(int argc, 
     char *argv[]
	) {

	float lv;
	arginfo ainfo;

	memset(&ainfo, 0, sizeof(arginfo));

	arginit(argc, argv, &ainfo);
  
	if(UsbOpen(ainfo.vender, ainfo.id, ainfo.num) < 0) {
		fprintf(stderr, "Frrio Open Faild\n");
		exit(1);
	}

	if(UsbInitalize(&ainfo) < 0) {
		fprintf(stderr, "Frrio Initialize Faild\n");
		exit(1);
	}

	if(ainfo.bscs == 0) {
		SetChannel(ainfo.channel);
	} else {
		SetChannel_Black(ainfo.channel);
	}

	if(ainfo.waittime > 0) {
		fprintf(stderr, "Wait %d Sec\n", ainfo.waittime);
		sleep(ainfo.waittime);
	}

	if(ainfo.bscs == 0) {
		if((lv = GetSignalLevel()) < 0) {
			fprintf(stderr, "Frrio GetSignalLevel Faild\n");
			exit(1);
		}
	} else {
		if((lv = GetSignalLevel_Black()) < 0) {
			fprintf(stderr, "Frrio GetSignalLevel Faild\n");
			exit(1);
		}
	}
	fprintf(stderr, "Signal:%f\n", lv);

	if(ainfo.rtpri >= 0) {
		PriorityControl(&ainfo);
	}
  
	readts(ainfo.outname, ainfo.recsec, ainfo.packetcount);

	/* ȥ̣ţ() */
	if(UsbProcLED(0)) {
		return -1;
	}

	UsbTerminate();
	UsbClose();

	return 0;
}

void UsbSendVendorRequest(uint16_t wValue, uint16_t wIndex)
{

	usb_control_req ureq;

	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_OUT|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest      = 0x01;
	ureq.wValue        = wValue;
	ureq.wIndex        = wIndex;
	ureq.wLength       = 0;
  
	if(UsbControlRequst(&ureq, NULL, 0) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}

}

/* USB */
int UsbInitalize(const arginfo *ainfo) {

	if(ainfo->bscs == 0) {
		/* ȥ */
		if(UsbProcFixInitA() < 0) {
			return -1;
		}
		/* ȥ */
		if(UsbProcFixInitB()< 0) {
			return -1;
		}
	} else {
		/* ȥ */
		if(UsbProcFixInitA_Black() < 0) {
			return -1;
		}
		/* ȥ */
		if(UsbProcFixInitB_Black() < 0) {
			return -1;
		}
	}
	/* ȥ꡼ǡ */
	if(UsbProcStreamInit() < 0) {
		return -1;
	}
	/* ȥ̣ţ() */
	if(UsbProcLED(1)) {
		return -1;
	}

	/* (o) from recfriio */
	return 0;
}

/*  */
int UsbProcFixInitA() {
	usb_control_req ureq;
	uint8_t rbuf;		
  
	UsbSendVendorRequest(0x0002,  0x0011);
	UsbSendVendorRequest(0x0000,  0x0011);

	memset(&ureq, 0, sizeof(usb_control_req));
  
	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest      = 0x02;
	ureq.wValue        = 0xA000;
	ureq.wIndex        = 0x00FE;
	ureq.wLength       = 1;
  
	if(UsbControlRequst(&ureq, &rbuf, 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		return -1;
	}

	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest      = 0x02;
	ureq.wValue        = 0x3000;
	ureq.wIndex        = 0x0080;
	ureq.wLength       = 1;
  
	if(UsbControlRequst(&ureq, &rbuf, 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		return -1;
	}

	UsbSendVendorRequest(0x0004, 0x0030);
	
	return 0;
}

/*  */
int UsbProcFixInitB() {
	UsbSendVendorRequest(0x3040, 0x01);
	UsbSendVendorRequest(0x3038, 0x04);
	UsbSendVendorRequest(0x3040, 0x05);
	UsbSendVendorRequest(0x3040, 0x07);
	UsbSendVendorRequest(0x304F, 0x0F);
	UsbSendVendorRequest(0x3021, 0x11);
	UsbSendVendorRequest(0x300B, 0x12);
	UsbSendVendorRequest(0x302F, 0x13);
	UsbSendVendorRequest(0x3031, 0x14);
	UsbSendVendorRequest(0x3002, 0x16);
	UsbSendVendorRequest(0x30C4, 0x21);
	UsbSendVendorRequest(0x3020, 0x22);
	UsbSendVendorRequest(0x3079, 0x2C);
	UsbSendVendorRequest(0x3034, 0x2D);
	UsbSendVendorRequest(0x3000, 0x2F);
	UsbSendVendorRequest(0x3028, 0x30);
	UsbSendVendorRequest(0x3031, 0x31);
	UsbSendVendorRequest(0x30DF, 0x32);
	UsbSendVendorRequest(0x3001, 0x38);
	UsbSendVendorRequest(0x3078, 0x39);
	UsbSendVendorRequest(0x3033, 0x3B);
	UsbSendVendorRequest(0x3033, 0x3C);
	UsbSendVendorRequest(0x3090, 0x48);
	UsbSendVendorRequest(0x3068, 0x51);
	UsbSendVendorRequest(0x3038, 0x5E);
	UsbSendVendorRequest(0x3000, 0x71);
	UsbSendVendorRequest(0x3008, 0x72);
	UsbSendVendorRequest(0x3000, 0x77);
	UsbSendVendorRequest(0x3021, 0xC0);
	UsbSendVendorRequest(0x3010, 0xC1);
	UsbSendVendorRequest(0x301A, 0xE4);
	UsbSendVendorRequest(0x301F, 0xEA);
	UsbSendVendorRequest(0x3000, 0x77);
	UsbSendVendorRequest(0x3000, 0x71);
	UsbSendVendorRequest(0x3000, 0x71);
	UsbSendVendorRequest(0x300C, 0x76);
  
	return 0;
}

/* ȥ꡼ */
int UsbProcStreamInit() {
	UsbSendVendorRequest(0x08, 0x33);
	UsbSendVendorRequest(0x40, 0x37);
	UsbSendVendorRequest(0x1F, 0x3A);
	UsbSendVendorRequest(0xFF, 0x3B);
	UsbSendVendorRequest(0x1F, 0x3C);
	UsbSendVendorRequest(0xFF, 0x3D);
	UsbSendVendorRequest(0x00, 0x38);
	UsbSendVendorRequest(0x00, 0x35);
	UsbSendVendorRequest(0x00, 0x39);
	UsbSendVendorRequest(0x00, 0x36);
	return 0;
}

int UsbProcLED(const uint8_t bType) {
  
	UsbSendVendorRequest(0x000F, 0x06);
	switch(bType)
    {
    case 0:
	{
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B,0x00);
		UsbSendVendorRequest(0x000F,0x00);

		UsbSendVendorRequest(0x0003,0x00);
		UsbSendVendorRequest(0x0007,0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0009, 0x00);
		UsbSendVendorRequest(0x000D, 0x00);
	}
	break;
    case 1:
	{
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0001, 0x00);
		UsbSendVendorRequest(0x0005, 0x00);
	}
	break;
    case 2:
	{
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);

		UsbSendVendorRequest(0x000B, 0x00);
		UsbSendVendorRequest(0x000F, 0x00);

		UsbSendVendorRequest(0x0003, 0x00);
		UsbSendVendorRequest(0x0007, 0x00);
		UsbSendVendorRequest(0x0001, 0x00);
		UsbSendVendorRequest(0x0005, 0x00);
	}
	break;
    case 3:
	{
		/* Ǥΰդ˵ʤäΤǥѥʤǤ
		   EP0ؤΥȥž
		   (40 01 0F 00 06 00 00 00)
		   λ(40 01 00 00 06 00 00 00)
		   ʥǡLEDȥݤΤǡäɵ˾ */
	}
	break;
    }
	UsbSendVendorRequest(0x0000, 0x06);

	return 0;
}

/* USBλ */
int UsbTerminate() {
	return 0;
}


/* ͥ(ͥϥӥåǥ) */
void SetChannel(const uint8_t bCh) {
	/*
	  if(bCh < 13 || bCh > 62)
	  errx(1, "invalid channel.");
	*/
  
	uint8_t sbuf[2][5];
	uint8_t rbuf[2][1];

	usb_control_req ureq;

	/* ͥǡ׻ */
	const uint32_t wChCode = 0x0E7F + 0x002A * (uint32_t)(bCh - 13);

	memset(sbuf, '\0', sizeof(sbuf));
	memset(rbuf, '\0', sizeof(rbuf));
  
	// 1ܥꥯ	
	sbuf[0][0] = 0xC0;
	sbuf[0][1] = (uint8_t)(wChCode >> 8);
	sbuf[0][2] = (uint8_t)(wChCode & 0xFF);
	sbuf[0][3] = 0xB2;
	sbuf[0][4] = 0x08;
  
	// 2ܥꥯ
	sbuf[1][0] = 0xC0;
	sbuf[1][1] = (uint8_t)(wChCode >> 8);
	sbuf[1][2] = (uint8_t)(wChCode & 0xFF);
	sbuf[1][3] = 0x9A;
	sbuf[1][4] = 0x50;
  
	// 1
	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_OUT|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest      = 0x03;
	ureq.wValue        = 0x3000;
	ureq.wIndex        = 0x00FE;
	ureq.wLength       = sizeof(sbuf[0]);

	if(UsbControlRequst(&ureq, sbuf[0], 5) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}
  
	// 2
	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_OUT|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest     = 0x03;
	ureq.wValue       = 0x3000;
	ureq.wIndex       = 0x00FE;
	ureq.wLength      = sizeof(sbuf[1]);
  
	if(UsbControlRequst(&ureq, sbuf[1], 5) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}
  
	// 3
	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest     = 0x02;
	ureq.wValue       = 0x3000;
	ureq.wIndex       = 0x00B0;
	ureq.wLength      = sizeof(rbuf[0]);

	if(UsbControlRequst(&ureq, rbuf[0], 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}

  
	// 4
	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest     = 0x02;
	ureq.wValue       = 0x3000;
	ureq.wIndex       = 0x0080;
	ureq.wLength      = sizeof(rbuf[1]);

	if(UsbControlRequst(&ureq, rbuf[1], 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}

}

float GetSignalLevel(int fd) {
	float sig_level = 0;
	uint8_t sigbuf[37];
	uint32_t dwHex;
	memset(sigbuf, '\0', sizeof(sigbuf));
  
	// 1
	usb_control_req ureq;

	memset(&ureq, 0, sizeof(usb_control_req));

	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest     = 0x02;
	ureq.wValue       = 0x3000;
	ureq.wIndex       = 0x0089;
	ureq.wLength      = sizeof(sigbuf);
  
	if(UsbControlRequst(&ureq, sigbuf, 37) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}

	dwHex = ((uint32_t)(sigbuf[2]) << 16) | ((uint32_t)(sigbuf[3]) << 8) | (uint32_t)(sigbuf[4]);
	// ٥׻
	if(!dwHex) {
		sig_level = 0.0f;
	} else {
		sig_level = (float)(68.4344993f - log10((double)dwHex) * 10.0 + pow(log10((double)dwHex) - 5.045f, (double)4.0f) / 4.09f);
	}
  
	return  sig_level;
}

/* ǡɤ߹߽ */
void readts(const char *fname, int recsec, int packetcount) {
  
	FILE *wfd = NULL;
	int endflag = 0;

	pthread_t eth;
	int stat;
	th_arg targ; 

	if(strcmp(fname, "-")) {
		wfd = fopen(fname , "wb+");
	} else {
		wfd = stdout;
	}  

	/* λƻ륹åɤ */
	targ.endflag = &endflag;
	targ.recsec  = recsec;
	stat = pthread_create(&eth, NULL, endcheck, &targ);
  
	UsbReciveIsoc(&endflag, wfd, packetcount);

	if(strcmp(fname, "-")) {
		fclose(wfd);
	}

	return;
}

/* ǡɤ߹߽ */
void *endcheck(void  *arg) {
	th_arg *targ = (th_arg*)arg;

	sleep(targ->recsec);
	*(targ->endflag) = 1;

	return NULL;
}

/* ǡɤ߹߽ */
void arginit(int argc, char *argv[], arginfo *ainfo) {

	int rtn;

	/* ܤν */
	ainfo->waittime = 30;
	ainfo->rtpri = -1;

	/* OS¸ΰ */
	ArgInit(ainfo);

	/* getoptexternѿ
	   optind = 0;
	   opterr = 0;
	   optopt = 0;
	   optarg = NULL;
	*/
  
	while((rtn = getopt(argc, argv, "p:v:i:n:w:r:b")) != -1) {
		switch(rtn) {
		case 'p':
			ainfo->packetcount = atoi(optarg);
			break;
		case 'v':
			ainfo->vender = optarg;
			break;
		case 'i':
			ainfo->id = optarg;
			break;
		case 'n':
			ainfo->num = atoi(optarg);
			break;
		case 'w':
			ainfo->waittime = atoi(optarg);
			break;
		case 'r':
			ainfo->rtpri = atoi(optarg);
			break;
		case 'b':
			ainfo->bscs = 1;
			break;
		}
	}
  
	/* ³Υå */
	if(optind >= argc) {
		PRINTUSAGE;
		exit(1);
	}
	ainfo->channel = atoi(argv[optind++]);

	if(optind >= argc) {
		PRINTUSAGE;
		exit(1);
	}
	ainfo->recsec = atoi(argv[optind++]);

	if(optind >= argc) {
		PRINTUSAGE;
		exit(1);
	}
	ainfo->outname = argv[optind++];

	if(optind < argc) {
		PRINTUSAGE;
		exit(1);
	}

}

/*  */
int UsbProcFixInitA_Black() {
	usb_control_req ureq;
	uint8_t rbuf;		
  
	UsbSendVendorRequest(0x0002,  0x0011);
	UsbSendVendorRequest(0x0000,  0x0011);

	memset(&ureq, 0, sizeof(usb_control_req));

	rbuf = 0xC1;
	ureq.bmRequestType = USB_DIR_OUT|USB_TYPE_VENDOR|USB_RECIP_DEVICE;;
	ureq.bRequest      = 0x03;
	ureq.wValue        = 0x3000;
	ureq.wIndex        = 0x00FE;
	ureq.wLength       = 1;
  
	if(UsbControlRequst(&ureq, &rbuf, 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		return -1;
	}

	memset(&ureq, 0, sizeof(usb_control_req));

	rbuf = 0x00;
	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest      = 0x02;
	ureq.wValue        = 0x3000;
	ureq.wIndex        = 0x0100;
	ureq.wLength       = 1;
  
	if(UsbControlRequst(&ureq, &rbuf, 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		return -1;
	}

	rbuf = 0x00;
	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest      = 0x02;
	ureq.wValue        = 0x3000;
	ureq.wIndex        = 0x0080;
	ureq.wLength       = 1;

	if(UsbControlRequst(&ureq, &rbuf, 1) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		return -1;
	}

	UsbSendVendorRequest(0x0004, 0x0030);
	
	return 0;
}

/*  */
int UsbProcFixInitB_Black() {
	UsbSendVendorRequest(0x3040, 0x01);
	UsbSendVendorRequest(0x3030, 0x04);
	UsbSendVendorRequest(0x3040, 0x05);
	UsbSendVendorRequest(0x3040, 0x07);
	UsbSendVendorRequest(0x30ff, 0x0f);
	UsbSendVendorRequest(0x3027, 0x11);
	UsbSendVendorRequest(0x302f, 0x13);
	UsbSendVendorRequest(0x3031, 0x14);
	UsbSendVendorRequest(0x3002, 0x16);
	UsbSendVendorRequest(0x3020, 0x22);
	UsbSendVendorRequest(0x3070, 0x24);
	UsbSendVendorRequest(0x3079, 0x2c);
	UsbSendVendorRequest(0x3034, 0x2d);
	UsbSendVendorRequest(0x3028, 0x30);
	UsbSendVendorRequest(0x3031, 0x31);
	UsbSendVendorRequest(0x3001, 0x38);
	UsbSendVendorRequest(0x3078, 0x39);
	UsbSendVendorRequest(0x3090, 0x48);
	UsbSendVendorRequest(0x3068, 0x51);
	UsbSendVendorRequest(0x3038, 0x5e);
	UsbSendVendorRequest(0x3021, 0xc0);
	UsbSendVendorRequest(0x3010, 0xc1);
	UsbSendVendorRequest(0x301a, 0xe4);
	UsbSendVendorRequest(0x301f, 0xea);
  
	return 0;
}

/* ­Τ̤ */
struct Black_Channel_Sequence {
	uint8_t byReqNo;
	uint16_t wValue;
	uint16_t wIndex;
	uint8_t bData;	/* 3byteΥǡ뤫 */
	uint8_t byData1;	/* ǡ */
	uint8_t byData2;	/* ǡ */
	uint8_t byData3;	/* ǡ */
};


static const uint32_t SETCHANNEL_COMMAND_INTERBAL = 5; /* SetChannelTimeout errorٱ(ms) */

/* ͥ */
void SetChannel_Black(uint32_t dwCh) {

	uint32_t dwSpace;
	struct Black_Channel_Sequence cmd[] = {
		/* Index(n-1) */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x00, 0x0A },	/* 0 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x01, 0x4C },	/* 1 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x02, 0x02 },	/* 2 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x03, 0x40 },	/* 3 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x04, 0x6C },	/* 4 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x05, 0x47 },	/* 5 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x06, 0x02 },	/* 6 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x07, 0x30 },	/* 7 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x08, 0x20 },	/* 8 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x09, 0xA2 },	/* 9 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0A, 0xF1 },	/* 10 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0B, 0x38 },	/* 11 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0C, 0xD0 },	/* 12 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0D, 0x10 },	/* 13 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0E, 0xB0 },	/* 14 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0F, 0x01 },	/* 15 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x00, 0x0A },	/* 16 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x01, 0x4C },	/* 17 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x02, 0x02 },	/* 18 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x03, 0x40 },	/* 19 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x04, 0x6C },	/* 20 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x05, 0x47 },	/* 21 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x06, 0x02 },	/* 22 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x07, 0x30 },	/* 23 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x08, 0x20 },	/* 24 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x09, 0xA2 },	/* 25 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0A, 0xF1 },	/* 26 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0B, 0x38 },	/* 27 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0C, 0xD0 },	/* 28 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0D, 0x10 },	/* 29 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0E, 0xB0 },	/* 30 */
		{ 0x03, 0x3200, 0x00FE, 1, 0xC0, 0x0F, 0x01 },	/* 31 */
		{ 0x01, 0x3210, 0x0001, 0, 0x00, 0x00, 0x00 },	/* 32 */
		{ 0x01, 0x3200, 0x0002, 0, 0x00, 0x00, 0x00 },	/* 33 */
		{ 0x01, 0x3200, 0x0003, 0, 0x00, 0x00, 0x00 },	/* 34 */
		{ 0x01, 0x3200, 0x0008, 0, 0x00, 0x00, 0x00 },	/* 35 */
		{ 0x01, 0x3259, 0x000C, 0, 0x00, 0x00, 0x00 },	/* 36 */
		{ 0x01, 0x32F2, 0x000D, 0, 0x00, 0x00, 0x00 },	/* 37 */
		{ 0x01, 0x3251, 0x000F, 0, 0x00, 0x00, 0x00 },	/* 38 */
		{ 0x01, 0x32B2, 0x0010, 0, 0x00, 0x00, 0x00 },	/* 39 */
		{ 0x01, 0x3204, 0x001A, 0, 0x00, 0x00, 0x00 },	/* 40 */
		{ 0x01, 0x3204, 0x001B, 0, 0x00, 0x00, 0x00 },	/* 41 */
		{ 0x01, 0x3240, 0x0038, 0, 0x00, 0x00, 0x00 },	/* 42 */
		{ 0x01, 0x3210, 0x0039, 0, 0x00, 0x00, 0x00 },	/* 43 */
		{ 0x01, 0x3208, 0x003C, 0, 0x00, 0x00, 0x00 },	/* 44 */
		{ 0x01, 0x3296, 0x003D, 0, 0x00, 0x00, 0x00 },	/* 45 */
		{ 0x01, 0x32D9, 0x003E, 0, 0x00, 0x00, 0x00 },	/* 46 */
		{ 0x01, 0x32B6, 0x003F, 0, 0x00, 0x00, 0x00 },	/* 47 */
		{ 0x01, 0x325D, 0x0040, 0, 0x00, 0x00, 0x00 },	/* 48 */
		{ 0x01, 0x3299, 0x0041, 0, 0x00, 0x00, 0x00 },	/* 49 */
		{ 0x01, 0x3258, 0x0044, 0, 0x00, 0x00, 0x00 },	/* 50 */
		{ 0x01, 0x3254, 0x0045, 0, 0x00, 0x00, 0x00 },	/* 51 */
		{ 0x01, 0x3230, 0x0046, 0, 0x00, 0x00, 0x00 },	/* 52 */
		{ 0x01, 0x3241, 0x0047, 0, 0x00, 0x00, 0x00 },	/* 53 */
		{ 0x01, 0x3216, 0x0049, 0, 0x00, 0x00, 0x00 },	/* 54 */
		{ 0x01, 0x3241, 0x004A, 0, 0x00, 0x00, 0x00 },	/* 55 */
		{ 0x01, 0x3241, 0x004B, 0, 0x00, 0x00, 0x00 },	/* 56 */
		{ 0x01, 0x325E, 0x004F, 0, 0x00, 0x00, 0x00 },	/* 57 */
		{ 0x01, 0x3204, 0x0050, 0, 0x00, 0x00, 0x00 },	/* 58 */
		{ 0x01, 0x32C0, 0x0051, 0, 0x00, 0x00, 0x00 },	/* 59 */
		{ 0x01, 0x328A, 0x0052, 0, 0x00, 0x00, 0x00 },	/* 60 */
		{ 0x01, 0x3213, 0x0053, 0, 0x00, 0x00, 0x00 },	/* 61 */
		{ 0x01, 0x3290, 0x0054, 0, 0x00, 0x00, 0x00 },	/* 62 */
		{ 0x01, 0x3292, 0x0055, 0, 0x00, 0x00, 0x00 },	/* 63 */
		{ 0x01, 0x326D, 0x0058, 0, 0x00, 0x00, 0x00 },	/* 64 */
		{ 0x01, 0x326F, 0x0059, 0, 0x00, 0x00, 0x00 },	/* 65 */
		{ 0x01, 0x322E, 0x005A, 0, 0x00, 0x00, 0x00 },	/* 66 */
		{ 0x01, 0x3223, 0x005B, 0, 0x00, 0x00, 0x00 },	/* 67 */
		{ 0x01, 0x3233, 0x005D, 0, 0x00, 0x00, 0x00 },	/* 68 */
		{ 0x01, 0x3204, 0x0087, 0, 0x00, 0x00, 0x00 },	/* 69 */
		{ 0x01, 0x3200, 0x008D, 0, 0x00, 0x00, 0x00 },	/* 70 */
		{ 0x01, 0x3000, 0x0077, 0, 0x00, 0x00, 0x00 },	/* 71 */
		{ 0x01, 0x3000, 0x0071, 0, 0x00, 0x00, 0x00 },	/* 72 */
		{ 0x01, 0x3000, 0x0071, 0, 0x00, 0x00, 0x00 },	/* 73 */
		{ 0x01, 0x3240, 0x008F, 0, 0x00, 0x00, 0x00 },	/* 74 */
		{ 0x01, 0x32F2, 0x0090, 0, 0x00, 0x00, 0x00 },	/* 75 */
		{ 0x01, 0x3207, 0x00A5, 0, 0x00, 0x00, 0x00 },	/* 76 */
		{ 0x01, 0x3207, 0x00A5, 0, 0x00, 0x00, 0x00 },	/* 77 */
		{ 0x01, 0x32FF, 0x00A3, 0, 0x00, 0x00, 0x00 },	/* 78 */
		{ 0x01, 0x32F7, 0x00A3, 0, 0x00, 0x00, 0x00 },	/* 79 */
		{ 0x01, 0x3040, 0x0001, 0, 0x00, 0x00, 0x00 },	/* 80 */
		{ 0x01, 0x3030, 0x0004, 0, 0x00, 0x00, 0x00 },	/* 81 */
		{ 0x01, 0x3040, 0x0005, 0, 0x00, 0x00, 0x00 },	/* 82 */
		{ 0x01, 0x3040, 0x0007, 0, 0x00, 0x00, 0x00 },	/* 83 */
		{ 0x01, 0x30FF, 0x000F, 0, 0x00, 0x00, 0x00 },	/* 84 */
		{ 0x01, 0x3027, 0x0011, 0, 0x00, 0x00, 0x00 },	/* 85 */
		{ 0x01, 0x302F, 0x0013, 0, 0x00, 0x00, 0x00 },	/* 86 */
		{ 0x01, 0x3031, 0x0014, 0, 0x00, 0x00, 0x00 },	/* 87 */
		{ 0x01, 0x3002, 0x0016, 0, 0x00, 0x00, 0x00 },	/* 88 */
		{ 0x01, 0x3020, 0x0022, 0, 0x00, 0x00, 0x00 },	/* 89 */
		{ 0x01, 0x3070, 0x0024, 0, 0x00, 0x00, 0x00 },	/* 90 */
		{ 0x01, 0x3079, 0x002C, 0, 0x00, 0x00, 0x00 },	/* 91 */
		{ 0x01, 0x3034, 0x002D, 0, 0x00, 0x00, 0x00 },	/* 92 */
		{ 0x01, 0x3028, 0x0030, 0, 0x00, 0x00, 0x00 },	/* 93 */
		{ 0x01, 0x3031, 0x0031, 0, 0x00, 0x00, 0x00 },	/* 94 */
		{ 0x01, 0x3001, 0x0038, 0, 0x00, 0x00, 0x00 },	/* 95 */
		{ 0x01, 0x3078, 0x0039, 0, 0x00, 0x00, 0x00 },	/* 96 */
		{ 0x01, 0x3090, 0x0048, 0, 0x00, 0x00, 0x00 },	/* 97 */
		{ 0x01, 0x3068, 0x0051, 0, 0x00, 0x00, 0x00 },	/* 98 */
		{ 0x01, 0x3038, 0x005E, 0, 0x00, 0x00, 0x00 },	/* 99 */
		{ 0x01, 0x3021, 0x00C0, 0, 0x00, 0x00, 0x00 },	/* 100 */
		{ 0x01, 0x3010, 0x00C1, 0, 0x00, 0x00, 0x00 },	/* 101 */
		{ 0x01, 0x301A, 0x00E4, 0, 0x00, 0x00, 0x00 },	/* 102 */
		{ 0x01, 0x301F, 0x00EA, 0, 0x00, 0x00, 0x00 },	/* 103 */
		{ 0x00, 0x0000, 0x0000, 0, 0x00, 0x00, 0x00 },	/* End */
	};
	
	/* 塼˥󥰥󥹲ʬ */
	static const uint8_t PllConfA[] = 
		{0x08, 0x08, 0x08, 0x08, 0x09, 0x09, 0x09, 0x0A, 0x0A, 0x0A, 
		 0x0A, 0x0C, 0x0C, 0x0D, 0x0D, 0x0D, 0x0E, 0x0E, 0x0E, 0x0F, 
		 0x0F, 0x0F, 0x10};	/* 1/17 +40 */
	static const uint8_t PllConfB[] = 
		{0x32, 0x32, 0x80, 0x80, 0x64, 0x64, 0x64, 0x00, 0x00, 0x4C, 
		 0x4C, 0x9A, 0xEA, 0x3A, 0x8A, 0xDA, 0x2A, 0x7A, 0xCA, 0x1A, 
		 0x6A, 0xBA, 0x0A};	/* 2/18 +40 */
	static const uint8_t PllConfC[] = 
		{0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 0x40, 
		 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		 0x00, 0x00, 0x00};	/* 75	+02 */
	static const uint8_t PllConfD[] = 
		{0x10, 0x11, 0x30, 0x31, 0x90, 0x91, 0x92, 0xD0, 0xD1, 0xF1, 
		 0xF2, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 
		 0x00, 0x00, 0x00};	/* 76	+02 */
	
	uint32_t dwChIndex;
	uint32_t dwXmitNum = 0;

	struct Black_Channel_Sequence tmp;
	uint8_t  buf[3];
	uint32_t buf_len;
		
	usb_control_req ureq;

	if ((dwCh >= 13U) && (dwCh <= 23U)) {
		/* BS */
		dwCh = dwCh - 13U;
		dwSpace =  0U;
	} else if ((dwCh >= 24U) && (dwCh <= 35U)){
		/* 110CS */
		dwCh = dwCh - 24U;
		dwSpace =  1U;
	} else {
		fprintf(stderr, "Channel error %d\n", dwCh);
		exit(1);
	}

	dwChIndex = dwSpace * 11UL + dwCh;

	/*  */
	/* BSAT-2c/3a TsID=4[BSchֹ(2byte)][TPֹ] */
	/* BS- 1  ȿ=11.727GHz 0832 TsID=4010 BSī */
	/* BS- 1  ȿ=11.727GHz 0832 TsID=4011 BS-i */
	/* BS- 3  ȿ=11.765GHz 0880 TsID=4030 WOWWOW */
	/* BS- 3  ȿ=11.765GHz 0880 TsID=4031 BSѥ */
	/* BS- 5                                   ʥǻ(WOWWOW) */
	/* BS- 7                                   ʥǻ(NHK BS-1) */
	/* BS- 9  ȿ=11.880GHz 0964 TsID=4090 BS11 */
	/* BS- 9  ȿ=11.880GHz 0964 TsID=4091 ͥ */
	/* BS- 9  ȿ=11.880GHz 0964 TsID=4092 TwellV */
	/* BS-11                                   ʥǻ(NHK BS-2) */
	/* BS-13  ȿ=11.957GHz 0A00 TsID=40D0 BSƥ */
	/* BS-13  ȿ=11.957GHz 0A00 TsID=40D1 BSե */
	/* BS-15  ȿ=11.996GHz 0A4C TsID=40F1 NHK BS1/BS2/˥塼/󥸥˥󥰥ȥ꡼ */
	/* BS-15  ȿ=11.996GHz 0A4C TsID=40F2 NHK BS-Hi */
	/* BS-17                                   ̤(BSAT-4ʹߤͽ) */
	/* BS-19                                   ̤(BSAT-4ʹߤͽ) */
	/* BS-21                                   ̤(BSAT-4ʹߤͽ) */
	/* BS-23                                   ̤(BSAT-4ʹߤͽ) */
	/* */
	/* NSAT-110 TsID=0000  */
	/* CSND2  ȿ=12.291GHz 0C9A TsID=0000 CS */
	/* CSND4  ȿ=12.331GHz 0CEA TsID=0000 CS */
	/* CSND6  ȿ=12.371GHz 0D3A TsID=0000 CS */
	/* CSND8  ȿ=12.411GHz 0D8A TsID=0000 CS */
	/* CSND10 ȿ=12.451GHz 0DDA TsID=0000 CS */
	/* CSND12 ȿ=12.491GHz 0E2A TsID=0000 CS */
	/* CSND14 ȿ=12.531GHz 0E7A TsID=0000 CS */
	/* CSND16 ȿ=12.571GHz 0ECA TsID=0000 CS */
	/* CSND18 ȿ=12.611GHz 0F1A TsID=0000 CS */
	/* CSND20 ȿ=12.651GHz 0F6A TsID=0000 CS */
	/* CSND22 ȿ=12.691GHz 0FBA TsID=0000 CS */
	/* CSND24 ȿ=12.731GHz 100A TsID=0000 CS */
	
	dwXmitNum = 0;
	for (;;dwXmitNum++) {
		tmp = cmd[dwXmitNum];
		if (tmp.byReqNo == 0x00) break; /* ü */

		/* ꥯȤκ */
		memset(buf, 0, sizeof(buf));
		buf_len = 0;
		
		if (tmp.bData == 1) {
			buf_len = 3;
			buf[0]  = tmp.byData1;
			buf[1]  = tmp.byData2;
			buf[2]  = tmp.byData3;
		}
		

		memset(&ureq, 0, sizeof(usb_control_req));

		ureq.bmRequestType = USB_DIR_OUT|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
		ureq.bRequest     = tmp.byReqNo;
		ureq.wValue       = tmp.wValue;
		ureq.wIndex       = tmp.wIndex;
		ureq.wLength      = buf_len;

		/* ѹ */
		switch(dwXmitNum){
		case 0:		
			/* ȿ1 ǡκǽbyteˤ */
			buf[2] = PllConfA[dwChIndex];	
			break;
		case 1:	
			/*  ȿ2 ǡκǽbyteˤ */
			buf[2] = PllConfB[dwChIndex];
			break;
		case 16:
			/* ȿ1 ǡκǽbyteˤ */
			buf[2] = PllConfA[dwChIndex];
			break;
		case 17:
			/* ȿ2 ǡκǽbyteˤ */
			buf[2] = PllConfB[dwChIndex];
			break;
		case 74:
			/* TsID1 wValueβByte */
			ureq.wValue = (ureq.wValue & 0xFF00) | ((uint16_t)PllConfC[dwChIndex]);
			break;
		case 75:
			/* TsID2 wValueβByte */
			ureq.wValue = (ureq.wValue & 0xFF00) | ((uint16_t)PllConfD[dwChIndex]);
			break;
		}
		
		/*  */
		if(UsbControlRequst(&ureq, buf, buf_len) < 0) {
			fprintf(stderr, "Control Transfer Faild\n");
			exit(1);
		}

		/* Τٱ򤫤ʤTimeout error롣 */
		usleep(SETCHANNEL_COMMAND_INTERBAL * 1000);
	}
}

/* ʥ٥ */
float GetSignalLevel_Black() {
	/* 䴰Ƕ */
	static const float afLevelTable[] =
		{
			24.07f,	/* 00	00	0		24.07dB */
			24.07f,	/* 10	00	4096	24.07dB */
			18.61f,	/* 20	00	8192	18.61dB */
			15.21f,	/* 30	00	12288	15.21dB */
			12.50f,	/* 40	00	16384	12.50dB */
			10.19f,	/* 50	00	20480	10.19dB */
			8.140f,	/* 60	00	24576	8.140dB */
			6.270f,	/* 70	00	28672	6.270dB */
			4.550f,	/* 80	00	32768	4.550dB */
			3.730f,	/* 88	00	34816	3.730dB */
			3.630f,	/* 88	FF	35071	3.630dB */
			2.940f,	/* 90	00	36864	2.940dB */
			1.420f,	/* A0	00	40960	1.420dB */
			0.000f	/* B0	00	45056	-0.01dB */
		};
	
	float fMixRate;
	usb_control_req ureq;

	uint8_t sigbuf[4];
	memset(sigbuf, '\0', sizeof(sigbuf));
	
	/* 1 */
	memset(&ureq, 0, sizeof(usb_control_req));
  
	ureq.bmRequestType = USB_DIR_IN|USB_TYPE_VENDOR|USB_RECIP_DEVICE;
	ureq.bRequest     = 0x02;
	ureq.wValue       = 0x3200;
	ureq.wIndex       = 0x00BA;
	ureq.wLength      = 4;
 
	/* ٥׵ */
	if(UsbControlRequst(&ureq, sigbuf, 4) < 0) {
		fprintf(stderr, "Control Transfer Faild\n");
		exit(1);
	}

	/* ٥׻ */
	if(sigbuf[2] <= 0x10U){
		/* 祯å */
		return 24.07f;
	} else if (sigbuf[2] >= 0xB0U) {
		/* Ǿå */
		return 0.0f;
	} else {
		/* 䴰 */ 
		fMixRate = (float)(((uint16_t)(sigbuf[2] & 0x0FU) << 8) | (uint16_t)sigbuf[3]) / 4095.0f;
		return afLevelTable[sigbuf[2] >> 4] * (1.0f - fMixRate) + afLevelTable[(sigbuf[2] >> 4) + 0x01U] * fMixRate;
	}
}
